import { Data, Effect, pipe } from 'effect';
import { dirname, relative } from 'path';
import { readFileSync } from './read-file-sync.js';
export class JsonParseError extends Data.TaggedClass('JsonParseError') {
}
export class JsonFile extends Data.TaggedClass('JsonFile') {
}
export function readJsonFileSync(io, filePath) {
    return pipe(Effect.Do, Effect.bind('json', () => readFileSync(io, filePath)), Effect.bind('contents', ({ json }) => Effect.try({
        try: () => JSON.parse(json),
        catch: (error) => new JsonParseError({ error, filePath, json }),
    })), Effect.map(({ contents, json }) => new JsonFile({
        contents,
        dirPath: dirname(filePath),
        filePath,
        json,
        shortPath: relative(io.process.cwd(), filePath),
    })));
}
